#!/usr/bin/env python3
"""
Network Deception Tool
Creates a network of decoy services to detect and trap attackers.
Parameters:
    - config: Configuration file for decoy services
    - mode: Operation mode (emulate, trap, aggressive)
Requires: admin/root privileges
"""
import socket
import threading
import logging
from datetime import datetime
import json
from pathlib import Path
import queue
import time
import yaml
import random
import subprocess
import platform
import psutil
import re
import tempfile
import paramiko

class DeceptionNetwork:
    def __init__(self, config_file=None, mode='emulate'):
        self.mode = mode
        self.config = self.load_config(config_file)
        self.is_windows = platform.system().lower() == 'windows'
        
        self.connections = queue.Queue()
        self.alerts = queue.Queue()
        self.stop_event = threading.Event()
        
        # Track attacker interactions
        self.attackers = {}
        self.credentials_used = set()
        
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("deception_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "deception.log"),
                logging.StreamHandler()
            ]
        )
        
    def load_config(self, config_file):
        """Load deception network configuration"""
        default_config = {
            'services': {
                'ftp': {
                    'port': 21,
                    'banner': '220 FTP server ready\r\n',
                    'emulation': True
                },
                'ssh': {
                    'port': 22,
                    'banner': 'SSH-2.0-OpenSSH_7.4\r\n',
                    'emulation': True
                },
                'telnet': {
                    'port': 23,
                    'banner': '\xff\xfb\x01\xff\xfb\x03\xff\xfd\x18\r\nlogin: ',
                    'emulation': True
                },
                'http': {
                    'port': 80,
                    'banner': 'HTTP/1.1 200 OK\r\nServer: Apache/2.4.6\r\n\r\n',
                    'emulation': True
                },
                'mysql': {
                    'port': 3306,
                    'banner': '5.7.22-0ubuntu0.16.04.1\x00',
                    'emulation': True
                }
            },
            'emulation': {
                'delay': 1.0,
                'max_connections': 50,
                'credentials': [
                    {'username': 'admin', 'password': 'admin123'},
                    {'username': 'root', 'password': 'password'},
                    {'username': 'user', 'password': '123456'}
                ]
            },
            'traps': {
                'fake_files': [
                    'passwords.txt',
                    'config.ini',
                    'backup.sql'
                ],
                'commands': [
                    'cat /etc/passwd',
                    'ls -la',
                    'whoami',
                    'netstat -an'
                ]
            }
        }
        
        if config_file and Path(config_file).exists():
            try:
                with open(config_file, 'r') as f:
                    custom_config = yaml.safe_load(f)
                    # Merge custom config with defaults
                    for category in default_config:
                        if category in custom_config:
                            default_config[category].update(custom_config[category])
            except Exception as e:
                logging.error(f"Error loading config file: {str(e)}")
                
        return default_config
        
    def create_fake_files(self):
        """Create fake sensitive files"""
        fake_dir = Path("fake_data")
        fake_dir.mkdir(exist_ok=True)
        
        files = {
            'passwords.txt': """
root:x:0:0:root:/root:/bin/bash
admin:x:1000:1000:admin:/home/admin:/bin/bash
user:x:1001:1001:user:/home/user:/bin/bash
""",
            'config.ini': """
[Database]
host = 192.168.1.100
port = 3306
username = admin
password = secretpass123

[API]
key = ab12cd34ef56gh78
endpoint = https://api.example.com
""",
            'backup.sql': """
CREATE DATABASE production;
USE production;
CREATE TABLE users (
    id INT PRIMARY KEY,
    username VARCHAR(50),
    password VARCHAR(100)
);
INSERT INTO users VALUES (1, 'admin', 'hash123');
"""
        }
        
        for filename, content in files.items():
            filepath = fake_dir / filename
            with open(filepath, 'w') as f:
                f.write(content)
                
        return fake_dir
        
    def create_fake_response(self, service, data):
        """Create fake response based on service and input"""
        if 'ssh' in service:
            return "Permission denied, please try again.\r\n"
            
        elif 'ftp' in service:
            if data.startswith(b'USER'):
                return "331 Please specify the password.\r\n"
            elif data.startswith(b'PASS'):
                return "530 Login incorrect.\r\n"
            else:
                return "500 Unknown command.\r\n"
                
        elif 'http' in service:
            if b'GET' in data:
                return """
HTTP/1.1 200 OK
Server: Apache/2.4.6
Content-Type: text/html

<html>
<head><title>Example Corp</title></head>
<body>
<h1>Welcome to Example Corp</h1>
<p>This page is under maintenance.</p>
</body>
</html>
"""
            else:
                return "HTTP/1.1 400 Bad Request\r\n"
                
        elif 'mysql' in service:
            return "\x00\x00\x00\x02\xff\xd7\x04Too many connections"
            
        return ""
        
    def handle_connection(self, client_socket, address, service):
        """Handle incoming connection to decoy service"""
        service_name = service.lower()
        
        try:
            # Log connection attempt
            self.connections.put({
                'timestamp': datetime.now().isoformat(),
                'service': service_name,
                'ip': address[0],
                'port': address[1]
            })
            
            # Track attacker
            if address[0] not in self.attackers:
                self.attackers[address[0]] = {
                    'first_seen': datetime.now().isoformat(),
                    'services': set(),
                    'attempts': 0,
                    'commands': []
                }
            
            self.attackers[address[0]]['services'].add(service_name)
            self.attackers[address[0]]['attempts'] += 1
            
            # Send banner
            if service_name in self.config['services']:
                banner = self.config['services'][service_name]['banner']
                client_socket.send(banner.encode())
                
            # Emulate service behavior
            if self.config['services'][service_name]['emulation']:
                while not self.stop_event.is_set():
                    try:
                        data = client_socket.recv(1024)
                        if not data:
                            break
                            
                        # Log received data
                        self.attackers[address[0]]['commands'].append(
                            data.decode(errors='ignore').strip()
                        )
                        
                        # Add delay to seem more realistic
                        time.sleep(self.config['emulation']['delay'])
                        
                        # Generate response
                        response = self.create_fake_response(service_name, data)
                        client_socket.send(response.encode())
                        
                        # Check for credential attempts
                        creds = self.extract_credentials(data)
                        if creds:
                            self.credentials_used.add(
                                f"{creds['username']}:{creds['password']}"
                            )
                            
                            # Alert on credential attempt
                            self.alerts.put({
                                'type': 'credential_attempt',
                                'service': service_name,
                                'ip': address[0],
                                'credentials': creds
                            })
                            
                    except socket.timeout:
                        continue
                        
        except Exception as e:
            logging.error(f"Error handling {service} connection from {address[0]}: {str(e)}")
        finally:
            client_socket.close()
            
    def extract_credentials(self, data):
        """Extract credentials from connection data"""
        try:
            data_str = data.decode(errors='ignore')
            
            # Common patterns for credentials
            patterns = {
                'ssh': r'(?P<username>[a-zA-Z0-9_-]+):(?P<password>.+)',
                'ftp': r'USER (?P<username>[^\r\n]+)|PASS (?P<password>[^\r\n]+)',
                'http': r'Authorization: Basic (?P<base64>[a-zA-Z0-9+/=]+)'
            }
            
            for service, pattern in patterns.items():
                match = re.search(pattern, data_str)
                if match:
                    if service == 'http' and 'base64' in match.groupdict():
                        import base64
                        try:
                            decoded = base64.b64decode(match['base64']).decode()
                            username, password = decoded.split(':')
                            return {'username': username, 'password': password}
                        except:
                            pass
                    else:
                        return match.groupdict()
                        
        except:
            pass
            
        return None
        
    def start_service(self, service, port):
        """Start individual decoy service"""
        try:
            server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            server.bind(('0.0.0.0', port))
            server.listen(5)
            server.settimeout(1)
            
            logging.info(f"Started decoy {service} service on port {port}")
            
            while not self.stop_event.is_set():
                try:
                    client, address = server.accept()
                    if len(threading.enumerate()) < self.config['emulation']['max_connections']:
                        threading.Thread(
                            target=self.handle_connection,
                            args=(client, address, service)
                        ).start()
                    else:
                        client.close()
                except socket.timeout:
                    continue
                    
        except Exception as e:
            logging.error(f"Error starting {service} service on port {port}: {str(e)}")
        finally:
            server.close()
            
    def monitor_alerts(self):
        """Monitor and handle alerts"""
        while not self.stop_event.is_set():
            try:
                alert = self.alerts.get(timeout=1)
                logging.warning(
                    f"Alert: {alert['type']} from {alert['ip']} "
                    f"on {alert['service']}"
                )
                
                # Additional actions based on alert type
                if alert['type'] == 'credential_attempt':
                    if self.mode in ['trap', 'aggressive']:
                        self.trigger_trap(alert['ip'])
                        
            except queue.Empty:
                continue
                
    def trigger_trap(self, ip):
        """Trigger trap for detected attacker"""
        logging.info(f"Triggering trap for {ip}")
        
        try:
            if self.mode == 'aggressive':
                # Create fake sensitive data
                fake_dir = self.create_fake_files()
                
                # Log trap activation
                self.alerts.put({
                    'type': 'trap_activated',
                    'ip': ip,
                    'fake_files': str(fake_dir)
                })
                
        except Exception as e:
            logging.error(f"Error triggering trap for {ip}: {str(e)}")
            
    def deploy_network(self):
        """Deploy deception network"""
        logging.info("Starting deception network")
        logging.info(f"Mode: {self.mode}")
        
        # Start services
        services = []
        for service, config in self.config['services'].items():
            if config['emulation']:
                thread = threading.Thread(
                    target=self.start_service,
                    args=(service, config['port'])
                )
                thread.start()
                services.append(thread)
                
        # Start alert monitor
        monitor = threading.Thread(target=self.monitor_alerts)
        monitor.start()
        
        try:
            while True:
                # Print status
                attacker_count = len(self.attackers)
                alert_count = sum(1 for _ in self.alerts.queue)
                print(f"\rActive attackers: {attacker_count}, "
                      f"Alerts: {alert_count}", end="")
                time.sleep(1)
                
        except KeyboardInterrupt:
            logging.info("\nStopping deception network...")
            self.stop_event.set()
            
            # Wait for threads to finish
            for thread in services:
                thread.join()
            monitor.join()
            
            # Save final state
            self.save_state()
            
    def save_state(self):
        """Save deception network state and results"""
        state = {
            'timestamp': datetime.now().isoformat(),
            'mode': self.mode,
            'attackers': {
                ip: {
                    'first_seen': data['first_seen'],
                    'services': list(data['services']),
                    'attempts': data['attempts'],
                    'commands': data['commands']
                }
                for ip, data in self.attackers.items()
            },
            'credentials_used': list(self.credentials_used)
        }
        
        # Get all alerts
        alerts = []
        while not self.alerts.empty():
            alerts.append(self.alerts.get())
        state['alerts'] = alerts
        
        # Save to file
        output_dir = Path("deception_logs")
        output_file = output_dir / f"state_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(output_file, 'w') as f:
            json.dump(state, f, indent=2)
            
        logging.info(f"State saved to {output_file}")
        
        # Print summary
        print("\nDeception Network Summary")
        print("=" * 50)
        print(f"Total Attackers: {len(state['attackers'])}")
        print(f"Credential Attempts: {len(state['credentials_used'])}")
        print(f"Total Alerts: {len(state['alerts'])}")
        
        print("\nTop Attacking IPs:")
        sorted_attackers = sorted(
            state['attackers'].items(),
            key=lambda x: x[1]['attempts'],
            reverse=True
        )
        for ip, data in sorted_attackers[:10]:
            print(f"\n{ip}:")
            print(f"- First seen: {data['first_seen']}")
            print(f"- Services targeted: {', '.join(data['services'])}")
            print(f"- Total attempts: {data['attempts']}")

if __name__ == "__main__":
    import argparse
    parser = argparse.ArgumentParser(description="Network Deception Tool")
    parser.add_argument("--config", help="Configuration file")
    parser.add_argument("--mode", choices=['emulate', 'trap', 'aggressive'],
                      default='emulate', help="Operation mode")
    args = parser.parse_args()
    
    deception = DeceptionNetwork(args.config, args.mode)
    deception.deploy_network()
